/******************************************************************************
 *                                                                            *
 * File:   execdir .c         Version  1.01            Date: 1996-08-03       *
 *                                                                            *
 * Copyright (C) 1993-1996 by kostis@acm.org (Kosta Kostis)                   *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *        - walk through a directory calling a program once for every file    *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * History:                                                                   *
 *     1996-08-03: KK V1.01                                                   *
 *        - added a few comments                                              *
 *     1993-04-28: KK V1.00                                                   *
 *        - initial coding                                                    *
 *                                                                            *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dir.h>
#include <dos.h>

#include "execdir.h"

/******************************************************************************

	Function:
		display copyright message

	Parameters:
		none

	Returns:
		nothing

 *****************************************************************************/

static  void    Hello (void)
{
	fprintf (stderr, MSG_COPYRIGHT) ;
	fprintf (stderr, MSG_VERSION, PRGNAM, VERSION) ;
}

/******************************************************************************

	Function:
		Walk through directory and fire up program once for every
		file matching the mask

	Parameters:
		struct  ffblk   *Dir    pointer through directory tree
		char    *Cmd            command to be called

	Returns:
		nothing

 *****************************************************************************/

static	void	ExecDir
(
	struct	ffblk	*Dir,
	char		*Cmd
)
{
	char		TmpCmd		[MAX_CMDLINE + 1] ;

	/**********************************************************************

		perform action in directory *once*

	 *********************************************************************/

	do
	{
		if (strncmp (Dir -> ff_name, ".", 1) != 0)
		{
			sprintf (TmpCmd, "%s %s", Cmd, Dir -> ff_name) ;
			system (TmpCmd) ;
		}
	}
	while (findnext (Dir) == 0) ;
}

/*****************************************************************************/

int     main
(
	int		argc,
	char		*argv		[]
)
{
	char		Cmd		[MAX_CMDLINE + 1] ;
	char		CurDir		[MAX_PATHNAME + 1] ;
	struct	ffblk	Dir ;
	int		i ;

	/**********************************************************************

		check syntax first

	 *********************************************************************/

	if (argc < 3)
	{
		Hello () ;
		fprintf (stderr, ERR_MSG_SYNTAX, PRGNAM) ;
		return (ERR_SYNTAX) ;
	}

	/**********************************************************************

		build command line with all parameters supplied

	 *********************************************************************/

	strcpy (Cmd, argv [2]) ;
	for (i = 3 ; i < argc ; ++i)
	{
		strcat (Cmd, " ") ;
		strcat (Cmd, argv [i]) ;
	}

	/**********************************************************************

		figure out current directory

	 *********************************************************************/

	if (getcwd (CurDir, MAX_PATHNAME) == (char *) NULL)
	{
		fprintf (stderr, ERR_MSG_CURDIR, PRGNAM) ;
		return (ERR_CURDIR) ;
	}

	/**********************************************************************

		add file mask

	 *********************************************************************/

	if (CurDir [strlen (CurDir) - 1] != '\\')
		strcat (CurDir, "\\") ;
	strcat (CurDir, argv [1]) ;

	/**********************************************************************

		process all files matching mask in current directory

	 *********************************************************************/

	if (findfirst (CurDir, &Dir, FA_ARCH) == 0)
		ExecDir (&Dir, Cmd) ;

	return (ERR_NONE) ;
}
